//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

public class AsyncData
{
	public Socket socket;
	public byte[] RecBytes = new byte[256];
}

public class AsyncSrv 
{	
	private static ManualResetEvent AcceptEvent = new ManualResetEvent(false);
	private static ManualResetEvent ReceiveEvent = new ManualResetEvent(false);	
	private static ManualResetEvent SendEvent = new ManualResetEvent(false);	

	private static AsyncData ad = new AsyncData();
	
	public static Socket GetListenerSocket(string host, int port) 
	{
		Socket s = null;
		IPHostEntry iphe = Dns.Resolve(host);
		foreach(IPAddress ipa in iphe.AddressList) 
		{
			IPEndPoint lep = new IPEndPoint(ipa, port);
			Socket ts = new Socket(lep.AddressFamily,
				                   SocketType.Stream, 
				                   ProtocolType.Tcp);
			try 
			{
				ts.Bind(lep);
				ts.Listen(2);
				s = ts;
				break;
			} 
			catch (ArgumentNullException ae)
			{
				Console.WriteLine("ArgumentNullException: " + ae.ToString());
			}
			catch (SocketException se)
			{				
				Console.WriteLine("SocketException: " + se.ToString());
			}
			catch (Exception e) 
			{
				Console.WriteLine("Connection failed: " + e.ToString());
			}
		}
		return s;
	}

	public static void Main() 
	{
		Socket srv = GetListenerSocket(Dns.GetHostName(), 5000);
		if(srv == null)
			return;

		Console.WriteLine("Now listening on port 5000\n");						
		
		while (true) 
		{
			AcceptEvent.Reset();
			ReceiveEvent.Reset();
			SendEvent.Reset();
			Console.WriteLine("calling BeginAccept");		
			srv.BeginAccept(new AsyncCallback(AcceptCallback), srv);		
			AcceptEvent.WaitOne();		
		}
	}

	private static void AcceptCallback(IAsyncResult result)
	{				
		Socket temp = (Socket) result.AsyncState;
		Socket a = temp.EndAccept(result);		

		Console.WriteLine("EndAccept() Socket.EndPoint: " + 
			a.RemoteEndPoint.ToString());				
		
		ad.socket = a;
		
		IAsyncResult recres = a.BeginReceive(ad.RecBytes, 
			0, 
			256, 
			0, 
			new AsyncCallback(ReceiveCallback), 
			ad);
		
		ReceiveEvent.WaitOne();		

		Byte[] byteSend = Encoding.ASCII.GetBytes("This is a test of the emergency broadcast system");

		IAsyncResult sendres = a.BeginSend(byteSend,
			0,
			byteSend.Length,
			0,
			new AsyncCallback(SendCallback),
			a);
		SendEvent.WaitOne();
				
		a.Close();
		AcceptEvent.Set();				

		return;
	}

	private static void ReceiveCallback(IAsyncResult result)
	{													
		int bytes = ad.socket.EndReceive(result);				
		
		Console.WriteLine("[AsyncSrv] ReceiveCallback: bytes = " + bytes);		

		if (bytes > 0)
		{
			string data = Encoding.ASCII.GetString(ad.RecBytes, 0, bytes);				
			Console.WriteLine("Received: [" + data + "]\n");		
		}
		
		ReceiveEvent.Set();
	}

	private static void SendCallback(IAsyncResult result) 
	{
		Socket socket = (Socket) result.AsyncState;

		int bytes = socket.EndSend(result);
		Console.WriteLine("Sent: " + bytes + " bytes");

		SendEvent.Set();
	}
}