//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Diagnostics;

namespace Bench
{
	public class AsyncData
	{
		public Socket socket;
		public byte[] buffer;
		public int byteOffset;
		public int byteCount;
	}
	/// <summary>
	/// Summary description for AsynchTcpTest.
	/// </summary>
	public class AsynchTcpTest : NetworkTest
	{
		private static ManualResetEvent ReceiveEvent = new ManualResetEvent(true);	
		private static ManualResetEvent SendEvent = new ManualResetEvent(false);	
		private byte[] writeBuffer;
		private byte[] readBuffer;
		public AsynchTcpTest(string server, int port, int numberOfPackets, int packetSize) :
			base(server, port, numberOfPackets, packetSize)
		{
		}
		public override void Start()
		{
			IPHostEntry host = Dns.GetHostByName(Server);
			Socket s = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
			IPEndPoint remoteEP = new IPEndPoint(host.AddressList[0], Port);

			s.Connect(remoteEP);

			// Build the buffers
			writeBuffer = new byte[PacketSize];
			readBuffer = new byte[PacketSize];

			// Fill the write buffer with some random data
			Random r = new Random();
			r.NextBytes(writeBuffer);

			// Create an asynchronous data structure for state.
			AsyncData ad = new AsyncData();
			// Initialize the state structure
			ad.buffer = writeBuffer;
			ad.socket = s;
			ad.byteCount = writeBuffer.Length;
			ad.byteOffset = 0;

			// Initialize the statistics.
			TransmitByteCount = 0;
			ReceivedByteCount = 0;

			// Send some dummy bytes
			// s.Send(writeBuffer, 0, 100, SocketFlags.None);
			// s.Receive(readBuffer, 0, 100, SocketFlags.None);

			// Perform the test
			StartCounters();
			int startTime = Environment.TickCount;
			for(int i = 0; i < NumberOfPackets; i++)
			{
				ReceiveEvent.WaitOne();
				IAsyncResult sendres = s.BeginSend(ad.buffer,
					                               0,
					                               ad.buffer.Length,
					                               0,
					                               new AsyncCallback(SendCallback),
					                               ad);
				SendEvent.WaitOne();
			}
			ReceiveEvent.WaitOne();
			int endTime = Environment.TickCount;
			StopCounters();

			Elapsed = endTime - startTime;

			// Untill this is fixed with RTM
			s.Close();
		}
		private void SendCallback(IAsyncResult result) 
		{
			AsyncData asd = (AsyncData) result.AsyncState;
			int bytes = asd.socket.EndSend(result);
			TransmitByteCount += bytes;
			asd.byteCount -= bytes;
			if(asd.byteCount > 0)
			{
				asd.byteOffset += bytes;
				IAsyncResult sendres = asd.socket.BeginSend(asd.buffer,
					                                        asd.byteOffset,
					                                        asd.byteCount,
					                                        SocketFlags.None,
					                                        new AsyncCallback(SendCallback),
					                                        asd);
			}
			else
			{
				ReceiveEvent.Reset();
				SendEvent.Set();

				AsyncData ard = new AsyncData();
				ard.buffer = readBuffer;
				ard.socket = asd.socket;
				ard.byteCount = readBuffer.Length;
				ard.byteOffset = 0;
				IAsyncResult recvres = ard.socket.BeginReceive(ard.buffer,
					                                           ard.byteOffset,
					                                           ard.byteCount,
					                                           SocketFlags.None,
					                                           new AsyncCallback(ReceiveCallback),
					                                           ard);
			}
		}
		private void ReceiveCallback(IAsyncResult result)
		{													
			AsyncData ad = (AsyncData) result.AsyncState;
			int bytes = ad.socket.EndSend(result);
			ReceivedByteCount += bytes;
			ad.byteCount -= bytes;
			if(ad.byteCount > 0)
			{
				ad.byteOffset += bytes;
				IAsyncResult recvres = ad.socket.BeginReceive(ad.buffer,
					                                          ad.byteOffset,
					                                          ad.byteCount,
					                                          SocketFlags.None,
					                                          new AsyncCallback(ReceiveCallback),
					                                          ad);
			}
			else
			{
				SendEvent.Reset();
				ReceiveEvent.Set();
			}
		}
	}
}
