//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Diagnostics;

namespace Bench
{
	/// <summary>
	/// Summary description for NetworkTest.
	/// </summary>
	public abstract class NetworkTest
	{
		public NetworkTest(string server, int port, int numberOfPackets, int packetSize)
		{
			this.server = server;
			this.port = port;
			this.numberOfPackets = numberOfPackets;
			this.packetSize = packetSize;
			elapsed = 0.0F;
			transmitByteCount = 0;
			receivedByteCount = 0;

			PerformanceCounterCategory pcCategory = new PerformanceCounterCategory("Network Interface");
			instanceNames = pcCategory.GetInstanceNames();

			performanceCounters = new PerformanceCounter[instanceNames.Length];
			startCounterValues = new float[instanceNames.Length];
			stopCounterValues = new float[instanceNames.Length];
			result = new float[instanceNames.Length];
			for(int i = 0; i < instanceNames.Length; i++)
			{
				performanceCounters[i] = new PerformanceCounter("Network Interface", "Bytes Total/sec", instanceNames[i]);
			}
		}
		public abstract void Start();
		public string Server
		{
			get
			{
				return server;
			}
		}
		public int Port
		{
			get
			{
				return port;
			}
		}
		public int NumberOfPackets
		{
			get
			{
				return numberOfPackets;
			}
		}
		public int PacketSize
		{
			get
			{
				return packetSize;
			}
		}
		public string [] InstanceNames
		{
			get
			{
				return instanceNames;
			}
		}
		public int TransmitByteCount
		{
			get
			{
				return transmitByteCount;
			}
			set
			{
				transmitByteCount = value;
			}
		}
		public int ReceivedByteCount
		{
			get
			{
				return receivedByteCount;
			}
			set
			{
				receivedByteCount = value;
			}
		}
		public float Elapsed
		{
			get
			{
				return elapsed;
			}
			set
			{
				elapsed = value;
			}
		}
		public float [] Rate
		{
			get
			{
				return result;
			}
		}
		public void StartCounters()
		{
			for(int i = 0;i < startCounterValues.Length;i++)
			{
				startCounterValues[i] = performanceCounters[i].NextValue();
			}
		}
		public void StopCounters()
		{
			for(int i = 0;i < stopCounterValues.Length;i++)
			{
				stopCounterValues[i] = performanceCounters[i].NextValue();
				result[i] = stopCounterValues[i];
			}
		}
		private string server;
		private int port;
		private int numberOfPackets;
		private int packetSize;
		private int transmitByteCount;
		private int receivedByteCount;
		private float elapsed;
		string [] instanceNames;
		PerformanceCounter [] performanceCounters;
		float [] startCounterValues;
		float [] stopCounterValues;
		float [] result;
	}
}
