//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Net;
using System.Net.Sockets;

namespace Bench
{
	/// <summary>
	/// Summary description for TcpClientTest.
	/// </summary>
	public class TcpClientTest : NetworkTest
	{
		public TcpClientTest(string server, int port, int numberOfPackets, int packetSize) :
			base(server, port, numberOfPackets, packetSize)
		{
		}
		public override void Start()
		{
			TcpClient client = new TcpClient();
			client.Connect(Server, Port);
			NetworkStream stream = client.GetStream();
			byte[] writeBuffer = new byte[PacketSize];
			byte[] readBuffer = new byte[PacketSize];
			int nTransmitted;
			int nReceived;
			int nTotalReceived;

			TransmitByteCount = 0;
			ReceivedByteCount = 0;

			{
				// Send some dummy bytes and make sure that 
				// we can send bytes and get the same bytes
				// back.
				Random r = new Random();
				r.NextBytes(writeBuffer);
				stream.Write(writeBuffer, 0, writeBuffer.Length);
				nTransmitted = writeBuffer.Length;
				for(nReceived = 0, nTotalReceived = 0; nTotalReceived < nTransmitted;)
				{
					nReceived = stream.Read(readBuffer, nReceived, nTransmitted - nTotalReceived);
					ReceivedByteCount += nReceived;
					nTotalReceived += nReceived;
				}
				for(int i = 0; i < writeBuffer.Length; i++)
				{
					if(writeBuffer[i] != readBuffer[i])
					{
						// This is a problem
						return;
					}
				}
			}

			// Perform the test
			StartCounters();
			int startTime = Environment.TickCount;
			for(int i = 0; i < NumberOfPackets; i++)
			{
				// nBytes = stream.Write(writeBuffer, 0, writeBuffer.Length);
				stream.Write(writeBuffer, 0, writeBuffer.Length);
				nTransmitted = writeBuffer.Length;
				TransmitByteCount += nTransmitted;
				for(nReceived = 0, nTotalReceived = 0; nTotalReceived < nTransmitted;)
				{
					nReceived = stream.Read(readBuffer, nReceived, nTransmitted - nTotalReceived);
					ReceivedByteCount += nReceived;
					nTotalReceived += nReceived;
				}
			}
			int endTime = Environment.TickCount;
			StopCounters();

			Elapsed = endTime - startTime;

			// Untill this is fixed with RTM
			client.Close();
		}
	}
}
