//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Diagnostics;
using System.Net;
using System.Net.Sockets;

namespace Bench
{
	/// <summary>
	/// Summary description for TcpTest.
	/// </summary>
	public class TcpTest : NetworkTest
	{
		public TcpTest(string server, int port, int numberOfPackets, int packetSize) :
			   base(server, port, numberOfPackets, packetSize)
		{
		}
		private Socket GetSocket(string host, int port) 
		{
			Socket s = null;
			IPHostEntry iphe = Dns.Resolve(host);
			// Loop through all addresses not just
			// the first address. 
			foreach(IPAddress ipa in iphe.AddressList) 
			{
				IPEndPoint ipe = new IPEndPoint(ipa, port);
		
				// Use AddressFamily the AddressFamily that
				// is part of the endpoint. 
				// AddressFamily.InterNetwork limits you to
				// just IPv4
				Socket ts = new Socket(ipe.AddressFamily,
					                   SocketType.Stream,
					                   ProtocolType.Tcp);
				try 
				{
					// Return the first socket that can be
					// we can connect to.
					ts.Connect(ipe);
					s = ts;
					break;
				} 
				catch (ArgumentNullException ae)
				{
					Console.WriteLine("ArgumentNullException: " + ae.ToString());
				}
				catch (SocketException se)
				{				
					Console.WriteLine("SocketException: " + se.ToString());
				}
				catch (Exception e) 
				{
					Console.WriteLine("Connection failed: " + e.ToString());
				}
			}
			return s;
		}
		public override void Start()
		{
			Socket s = GetSocket(Server, Port);
			if(s == null)
				return;

			byte[] writeBuffer = new byte[PacketSize];
			byte[] readBuffer = new byte[PacketSize];
			int nTransmitted;
			int nReceived;
			int nTotalReceived;

			TransmitByteCount = 0;
			ReceivedByteCount = 0;

			// Send some dummy bytes
			s.Send(writeBuffer, 0, 100, SocketFlags.None);
			s.Receive(readBuffer, 0, 100, SocketFlags.None);

			// Perform the test
			int counterStart = Environment.TickCount;
			StartCounters();
			int counterStop = Environment.TickCount;
			Debug.WriteLine("Counter start " + Convert.ToString(counterStop - counterStart));
			int startTime = Environment.TickCount;
			for(int i = 0; i < NumberOfPackets; i++)
			{
				nTransmitted = s.Send(writeBuffer, 0, writeBuffer.Length, SocketFlags.None);
				TransmitByteCount += nTransmitted;
				for(nReceived = 0, nTotalReceived = 0; nTotalReceived < nTransmitted;)
				{
					nReceived = s.Receive(readBuffer, nReceived, nTransmitted - nTotalReceived, SocketFlags.None);
					ReceivedByteCount += nReceived;
					nTotalReceived += nReceived;
				}
			}
			int endTime = Environment.TickCount;
			counterStart = Environment.TickCount;
			StopCounters();
			counterStop = Environment.TickCount;
			Debug.WriteLine("Counter stop " + Convert.ToString(counterStop - counterStart));

			Elapsed = endTime - startTime;

			// Untill this is fixed with RTM
			s.Close();
		}
	}
}
