//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Net;
using System.Net.Sockets;

namespace Bench
{
	/// <summary>
	/// Summary description for UdpClientTest.
	/// </summary>
	public class UdpClientTest : NetworkTest
	{
		public UdpClientTest(string server, int port, int numberOfPackets, int packetSize) :
			base(server, port, numberOfPackets, packetSize)
		{
		}
		public override void Start()
		{
			UdpClient client = new UdpClient();
 
			IPAddress address = Dns.Resolve(Server).AddressList[0];
			IPEndPoint remoteEp = new IPEndPoint(address, Port);

			byte[] writeBuffer = new byte[PacketSize];
			int nTransmitted;
			int nTotalReceived;

			TransmitByteCount = 0;
			ReceivedByteCount = 0;

			// Send some dummy bytes
			{
				client.Send(writeBuffer, 100, Server, Port);
				byte [] readBuffer = client.Receive(ref remoteEp);
			}

			// Perform the test
			StartCounters();
			int startTime = Environment.TickCount;
			for(int i = 0; i < NumberOfPackets; i++)
			{
				nTransmitted = client.Send(writeBuffer, writeBuffer.Length, Server, Port);
				TransmitByteCount += nTransmitted;
				for(nTotalReceived = 0; nTotalReceived < nTransmitted;)
				{
					byte [] readBuffer = client.Receive(ref remoteEp);
					ReceivedByteCount += readBuffer.Length;
					nTotalReceived += readBuffer.Length;
				}
			}
			int endTime = Environment.TickCount;
			StopCounters();

			Elapsed = endTime - startTime;

			// Untill this is fixed with RTM
			client.Close();
		}
	}
}
