//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Net;
using System.Net.Sockets;

namespace Bench
{
	/// <summary>
	/// Summary description for UdpTest.
	/// </summary>
	public class UdpTest : NetworkTest
	{
		public UdpTest(string server, int port, int numberOfPackets, int packetSize) :
			base(server, port, numberOfPackets, packetSize)
		{
		}
		public override void Start()
		{
			Socket client = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);

			byte[] writeBuffer = new byte[PacketSize];
			byte[] readBuffer = new byte[PacketSize];
			int nTransmitted;
			int nReceived;
			int nTotalReceived;

			IPAddress address = Dns.Resolve(Server).AddressList[0];
			IPEndPoint remoteEP = new IPEndPoint(address, Port);

			int nBytes;
			// Send dummy bytes
			nBytes = client.SendTo(writeBuffer, 0, 100, SocketFlags.None, remoteEP);
			EndPoint trEP = (EndPoint)remoteEP;
			nBytes = client.ReceiveFrom(readBuffer, 0, 100, SocketFlags.None, ref trEP);

			// Perform the test
			StartCounters();
			int startTime = Environment.TickCount;
			for(int i = 0; i < NumberOfPackets; i++)
			{
				// nBytes = stream.Write(writeBuffer, 0, writeBuffer.Length);
				nTransmitted = client.SendTo(writeBuffer, 0, PacketSize, SocketFlags.None, remoteEP);
				TransmitByteCount += nTransmitted;
				for(nReceived = 0, nTotalReceived = 0; nTotalReceived < nTransmitted;)
				{
					nReceived = client.ReceiveFrom(readBuffer, nReceived, nTransmitted - nTotalReceived, SocketFlags.None, ref trEP);
					ReceivedByteCount += nReceived;
					nTotalReceived += nReceived;
				}
			}
			int endTime = Environment.TickCount;
			StopCounters();

			Elapsed = endTime - startTime;

			// Untill this is fixed with RTM
			client.Close();
		}
	}
}
