//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the UNMANAGEDNETWORKING_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// UNMANAGEDNETWORKING_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef UNMANAGEDNETWORKING_EXPORTS
#define UNMANAGEDNETWORKING_API __declspec(dllexport)
#else
#define UNMANAGEDNETWORKING_API __declspec(dllimport)
#endif
extern "C"
{
	UNMANAGEDNETWORKING_API int Start();
	UNMANAGEDNETWORKING_API int TransmitByteCount();
	UNMANAGEDNETWORKING_API int ReceivedByteCount();
	UNMANAGEDNETWORKING_API double Elapsed();
	UNMANAGEDNETWORKING_API void Stop();
};

HRESULT ReportSocketError(const std::string& comment, std::string& strMessage);

class NetworkTest
{
public:
	NetworkTest(const std::string& _server, int _port, int _numberOfPackets, int _packetSize) :
		        server(_server),
				port(_port),
				numberOfPackets(_numberOfPackets),
				packetSize(_packetSize)
	{
		elapsed = 0.0F;
		transmitByteCount = 0;
		receivedByteCount = 0;
		socket = INVALID_SOCKET;
		memset(&sin, 0, sizeof(sin));
	}
	virtual ~NetworkTest()
	{
		if(socket != INVALID_SOCKET)
		{
			::closesocket(socket);
			socket = INVALID_SOCKET;
		}
	}
	virtual int Start() = 0;
	std::string GetServer() const
	{
		return server;
	}
	int GetPort() const
	{
		return port;
	}
	int GetNumberOfPackets() const
	{
		return numberOfPackets;
	}
	int GetPacketSize() const
	{
		return packetSize;
	}
	int GetTransmitByteCount() const
	{
		return transmitByteCount;
	}
	void SetTransmitByteCount(int value)
	{
		transmitByteCount = value;
	}
	int GetReceivedByteCount() const
	{
		return receivedByteCount;
	}
	void SetReceivedByteCount(int value)
	{
		receivedByteCount = value;
	}
	double GetElapsed()const
	{
		return elapsed;
	}
	void SetElapsed(double value)
	{
		elapsed = value;
	}
protected:
	std::string server;
	int port;
	int numberOfPackets;
	int packetSize;
	int transmitByteCount;
	int receivedByteCount;
	double elapsed;
	SOCKET socket;
	SOCKADDR_IN sin;
};

