//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Security.Permissions;

class Listener
{
	private static void AssertAndGo()
	{
		Console.WriteLine("Enable from enkidu to local 5000...");
		SocketPermission sp = new SocketPermission(PermissionState.None);
		sp.AddPermission(NetworkAccess.Connect,
			TransportType.Tcp,
			"gilgamesh",
			SocketPermission.AllPorts);
		sp.AddPermission(NetworkAccess.Accept,
			TransportType.Tcp,
			"*.*.*.*",
			5000);
		sp.Assert();

		TcpListener client = new TcpListener(5000);
		client.Start();
		Socket s = client.AcceptSocket();

		ProcessRequests(s);

		s.Close();

		client.Stop();
	}

	private static void DenyThenAssertThenGo()
	{
		Console.WriteLine("Disable all...");
		SocketPermission sp = new SocketPermission(PermissionState.Unrestricted);
		sp.Deny();
		AssertAndGo();
	}

	private static void ProcessRequests(Socket s)
	{
		byte[] responseData = new byte[128];
		bool continueProcessing = true;

		try
		{
			IPEndPoint remoteEndPoint = (IPEndPoint)s.RemoteEndPoint;
			IPAddress remoteAddress = remoteEndPoint.Address;
			IPHostEntry remoteHostEntry = Dns.GetHostByAddress(remoteAddress);
			Console.WriteLine("Accepted call from remote site ({0})...", remoteHostEntry.HostName);
			SocketPermission sp = new SocketPermission(NetworkAccess.Connect,
				TransportType.Tcp,
				remoteHostEntry.HostName,
				remoteEndPoint.Port);
			sp.Demand();
		}
		catch(Exception e)
		{
			Console.WriteLine("Permission error: {0}", e.ToString());
			return;
		}

		Console.WriteLine("Attempting read/write...");

		while(continueProcessing)
		{	
			try
			{
				int bytesRead = s.Receive(responseData);
				// string message = Encoding.ASCII.GetString(responseData, 0, bytesRead);
				string message = Encoding.Unicode.GetString(responseData, 0, bytesRead);
				if(message == "quit")
				{
					continueProcessing = false;
					break;
				}
				else
				{
					Console.WriteLine("The message received was " + message);
				}
			}
			catch(Exception ex)
			{
				Console.WriteLine(ex.ToString());
			}
		}
	}

	public static void Main()
	{
		Console.WriteLine("Ready to process requests...");
		DenyThenAssertThenGo();
	}
}
