//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Text;
using System.IO;
using System.Net;
using System.Net.Sockets;

class Sender
{

	public static void Main(string[] args)
	{	
		string address = "localhost";
		if(args.Length == 1)
			address = args[0];

		SendMessage(address);
	}

	public static Socket GetSocket(string host, int port) 
	{
		Socket s = null;
		IPHostEntry iphe = Dns.Resolve(host);
		// Loop through all addresses not just
		// the first address. 
		foreach(IPAddress ipa in iphe.AddressList) 
		{
			IPEndPoint ipe = new IPEndPoint(ipa, port);
		
			// Use AddressFamily the AddressFamily that
			// is part of the endpoint. 
			// AddressFamily.InterNetwork limits you to
			// just IPv4
			Socket ts = new Socket(ipe.AddressFamily,
				                   SocketType.Stream,
				                   ProtocolType.Tcp);
			try 
			{
				// Return the first socket that can be
				// we can connect to.
				ts.Connect(ipe);
				s = ts;
				break;
			} 
			catch (ArgumentNullException ae)
			{
				Console.WriteLine("ArgumentNullException: " + ae.ToString());
			}
			catch (SocketException se)
			{				
				Console.WriteLine("SocketException: " + se.ToString());
			}
			catch (Exception e) 
			{
				Console.WriteLine("Connection failed: " + e.ToString());
			}
		}
		return s;
	}

	public static void SendMessage(string server)
	{
		Socket s = GetSocket(server, 5000);
		if(s == null)
			return;
		bool continueSending = true;
		while(continueSending)
		{		
			Console.WriteLine("Enter the message to send. Type 'quit' to exit.");
			string message = Console.ReadLine();
			// byte[] buffer = Encoding.ASCII.GetBytes(message);
			byte[] buffer = Encoding.Unicode.GetBytes(message);
			// This sends the whole buffer. Look for other overloads.
			s.Send(buffer);													
			if(message == "quit")
			{
				continueSending = false;
			}	
		}
		s.Close();
	}
}
