//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Net;
using System.Text;
using System.IO;
using System.Collections.Specialized;

namespace HTMLViewer
{
	/// <summary>
	/// Summary description for HTMLPageGet.
	/// </summary>
	public class HTMLPageGet
	{
		public enum SourceType
		{
			Unknown = 0,
			Html,
			File
		}

		private WebRequest request;
		private WebResponse response;
		private StringBuilder strSource = new StringBuilder();
		public HTMLPageGet(string url)
		{
			try
			{
				request = WebRequest.Create(url);
				response = request.GetResponse();
				// get the stream of data
				StreamReader sr = new StreamReader(response.GetResponseStream(), Encoding.ASCII);
				string strTemp;
				while ((strTemp = sr.ReadLine()) != null)
				{
					strSource.Append(strTemp + "\r\n");					
				}
				sr.Close();
				if(request is HttpWebRequest)
				{
					HttpWebRequest httpRequest = (HttpWebRequest)request;
				}
				else if(request is FileWebRequest)
				{
					FileWebRequest fileRequest = (FileWebRequest)request;
				}
			}
			catch (WebException ex) 
			{
				strSource.Append(ex.Message);
			}			
		}
		public string Source
		{
			get
			{
				return strSource.ToString();
			}
		}
		public SourceType Type
		{
			get
			{
				if(request is HttpWebRequest)
				{
					return SourceType.Html;
				}
				else if(request is FileWebRequest)
				{
					return SourceType.File;
				}
				else
				{
					return SourceType.Unknown;
				}
			}
		}
		public NameValueCollection SourceProperties
		{
			get
			{
				NameValueCollection c = new NameValueCollection();

				if(response is HttpWebResponse)
				{
					HttpWebResponse r = (HttpWebResponse)response;
					c.Add("Address", r.ResponseUri.AbsoluteUri);
					c.Add("CharacterSet", r.CharacterSet);
					c.Add("Cookies", Convert.ToString(r.Cookies.Count));
					c.Add("ContentEncoding", r.ContentEncoding);
					c.Add("ContentLength", Convert.ToString(r.ContentLength));
					c.Add("ContentType", r.ContentType);
					c.Add("LastModified", Convert.ToString(r.LastModified));
					c.Add("Method", r.Method);
					c.Add("ProtocolVersion", r.ProtocolVersion.ToString());
					c.Add("Server", r.Server);
					c.Add("StatusCode", Convert.ToString(r.StatusCode));
					c.Add("StatusDescription", r.StatusDescription);
				}
				else if(request is FileWebRequest)
				{
				}
				return c;
			}
		}
	}
}
