//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Net;
using System.Text;
using System.IO;
using System.Threading;

public class AsyncResponseData
{
	public AsyncResponseData (WebRequest webRequest)
	{
		this.webRequest = webRequest;
		this.responseDone = new ManualResetEvent(false);
	}
	public WebRequest Request
	{
		get
		{
			return webRequest;
		}
	}
	public ManualResetEvent ResponseEvent
	{
		get
		{
			return responseDone;
		}
	}
	private WebRequest webRequest;
	private ManualResetEvent responseDone;
}
public class AsyncReadData
{
	public AsyncReadData (Stream stream)
	{
		this.stream = stream;
		bytesRead = -1;
		readDone = new ManualResetEvent(false);
		page = new StringBuilder();
	}
	public Stream GetStream
	{
		get
		{
			return stream;
		}
	}
	public byte [] Buffer
	{
		get
		{
			return buffer;
		}
		set
		{
			buffer = value;
		}
	}
	public int ReadCount
	{
		get
		{
			return bytesRead;
		}
		set
		{
			bytesRead = value;
		}
	}
	public StringBuilder Page
	{
		get
		{
			return page;
		}
		set
		{
			page = value;
		}
	}
	public ManualResetEvent ReadEvent
	{
		get
		{
			return readDone;
		}
	}
	private Stream stream;
	private byte[] buffer = new byte[4096];
	private int bytesRead = 0;
	private StringBuilder page;
	private ManualResetEvent readDone;
}

public class WebAsynch
{
	public static void Main(string[] args) 
	{
		try
		{
			string address = "http://localhost/QuickStart/HowTo/";
			if(args.Length == 1)
				address = args[0];

			WebRequest request = WebRequest.Create(address);

			AsyncResponseData ad = new AsyncResponseData (request);
			IAsyncResult responseResult = request.BeginGetResponse(new AsyncCallback(ResponseCallback),
																   ad);
			ad.ResponseEvent.WaitOne();
		}
		catch(Exception ex)
		{
			Console.WriteLine(ex.ToString());
		}		
	}
	private static void ResponseCallback(IAsyncResult result)
	{
		AsyncResponseData ar = (AsyncResponseData)result.AsyncState;
		WebRequest request = ar.Request;
		WebResponse response = request.EndGetResponse(result);
		Stream stream = response.GetResponseStream();
		AsyncReadData ad = new AsyncReadData(stream);
		IAsyncResult readResult = stream.BeginRead(ad.Buffer,
			                                       0,
			                                       ad.Buffer.Length,
			                                       new AsyncCallback(ReadCallback),
			                                       ad);
		ad.ReadEvent.WaitOne();
		ar.ResponseEvent.Set();
	}
	private static void ReadCallback(IAsyncResult result)
	{
		AsyncReadData ad = (AsyncReadData)result.AsyncState;
		Stream stream = ad.GetStream;
		int bytesRead = stream.EndRead(result);
		if(bytesRead == 0)
		{
			// The end of the read
			Console.WriteLine(ad.Page.ToString());
			ad.ReadEvent.Set();
		}
		else
		{
			ad.Page.Append(Encoding.ASCII.GetString(ad.Buffer, 0, bytesRead));
			IAsyncResult readResult = stream.BeginRead(ad.Buffer,
				                                       0,
				                                       ad.Buffer.Length,
				                                       new AsyncCallback(ReadCallback),
				                                       ad);
		}
	}
}
